/*
*/

#include "capture.h"
#include "device.h"

#ifdef ALLOC_PRAGMA
#pragma code_seg("PAGE")
#endif // ALLOC_PRAGMA


/* CapturePin */

// Public methods



// Public static methods

/**
 * Create a new capture pin. This is the creation dispatch for
 * the audio capture pin.
 * @param ksPin The pin being created
 * @param irp The creation Irp
 * @return NT status code.
 */
NTSTATUS
CapturePin::DispatchCreate(KSPIN *ksPin, IRP *irp)
{
    PAGED_CODE();

    DbgPrint("CapturePin::DispatchCreate\n");

    (void)irp;

    CapturePin *capPin = new (NonPagedPool) CapturePin(ksPin);
    if (!capPin) {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    // Add the item to the object bag if we we were successful.
    // Whenever the pin closes, the bag is cleaned up and we will be
    // freed.
    NTSTATUS status = ::KsAddItemToObjectBag(ksPin->Bag,
                                             reinterpret_cast<PVOID>(capPin),
                                             reinterpret_cast<PFNKSFREE>(CapturePin::Cleanup));
    if (NT_SUCCESS(status)) {
        ksPin->Context = reinterpret_cast<PVOID>(capPin);
    } else {
        delete capPin;
    }

    //

    return status;
}

/**
 * This is the set device state dispatch for the pin. The routine bridges
 * to SetState() in the context of the CapturePin.
 * @param ksPin The pin..
 * @param toKsState
 * @param fromKsState
 * @return NT status code.
 */
NTSTATUS
CapturePin::DispatchSetState(KSPIN *ksPin, KSSTATE toKsState, KSSTATE fromKsState)
{
    PAGED_CODE();

    DbgPrint("CapturePin::DispatchSetState\n");

    CapturePin *capPin = reinterpret_cast<CapturePin *>(ksPin->Context);
    return capPin->SetState(toKsState, fromKsState);
}

/**
 * This is the set data format dispatch for the capture pin. It is called
 * in two circumstances:
 * # before Pin's creation dispatch has been made to verify that
 * ksPin->ConnectionFormat is an acceptable format for the range
 * DataRange. In this case oldKsFormat is NULL.
 * # after Pin's creation dispatch has been made and an initial format
 * selected in order to change the format for the pin. In this case,
 * oldKsFormat will not be NULL.
 * Validate that the format is acceptible and perform the actions necessary
 * to change format if appropriate.
 * @param ksPin The pin this format is being set on. The format itself will be in
 * ksPin->ConnectionFormat.
 * @param oldKsFormat The previous format used on this pin. If this is NULL, it is an
 * indication that Pin's creation dispatch has not yet been made and
 * that this is a request to validate the initial format and not to
 * change formats.
 * @param oldKsAttributeList The old attribute list for the prior format
 * @param ksDtaRange A range out of our list of data ranges which was determined to be
 * at least a partial match for ksPin->ConnectionFormat. If the format
 * there is unacceptable for the range, STATUS_NO_MATCH should be returned.
 * @param ksAttributeRange The attribute range.
 * @return NT status code.
 */
NTSTATUS
CapturePin::DispatchSetFormat(KSPIN *ksPin,
                              KSDATAFORMAT *oldKsFormat, KSMULTIPLE_ITEM *oldKsAttributeList,
                              const KSDATARANGE *ksDtaRange, const KSATTRIBUTE_LIST *ksAttributeRange)
{
    PAGED_CODE();

    DbgPrint("CapturePin::DispatchSetFormat\n");

    (void)ksPin;
    (void)oldKsFormat;
    (void)oldKsAttributeList;
    (void)ksDtaRange;
    (void)ksAttributeRange;

    NTSTATUS status = STATUS_SUCCESS;
    return status;
}

/**
 * This is the processing dispatch for the capture pin.  The routine
 * bridges to Process() in the context of the CapturePin.
 * @param
 * @return NT status code.
 */
NTSTATUS
CapturePin::DispatchProcess(KSPIN *ksPin)
{
    PAGED_CODE();

    DbgPrint("CapturePin::DispatchProcess\n");

    CapturePin *capPin = reinterpret_cast<CapturePin *>(ksPin->Context);
    return capPin->Process();
}

/**
 * This routine handles audio pin intersection queries by determining the
 * intersection between two data ranges.
 * @param ksFilter Contains a void pointer to the  filter structure.
 * @param irp Contains a pointer to the data intersection property request.
 * @param ksPinInstance Contains a pointer to a structure indicating the pin in question.
 * @param ksCallerDataRange Contains a pointer to one of the data ranges supplied by the client
 * in the data intersection request. The format type, subtype and
 * specifier are compatible with the DescriptorDataRange.
 * @param ksDescriptorDataRange Contains a pointer to one of the data ranges from the pin descriptor
 * for the pin in question. The format type, subtype and specifier are
 * compatible with the CallerDataRange.
 * @param bufferSize Contains the size in bytes of the buffer pointed to by the Data
 * argument. For size queries, this value will be zero.
 * @param data Optionally contains a pointer to the buffer to contain the data
 * format structure representing the best format in the intersection
 * of the two data ranges.  For size queries, this pointer will be NULL.
 * @param dataSize Contains a pointer to the location at which to deposit the size
 * of the data format. This information is supplied by the function
 * when the format is actually delivered and in response to size queries.
 * @return NT status code.
 */
NTSTATUS
CapturePin::IntersectHandler(KSFILTER *ksFilter, IRP *irp, KSP_PIN *ksPinInstance,
                             KSDATARANGE *ksCallerDataRange, KSDATARANGE *ksDescriptorDataRange,
                             ULONG bufferSize, VOID *data, ULONG *dataSize)
{
    PAGED_CODE();

    DbgPrint("CapturePin::IntersectHandler\n");

    ASSERT(ksFilter);
    ASSERT(irp);
    ASSERT(ksPinInstance);
    ASSERT(ksCallerDataRange);
    ASSERT(ksDescriptorDataRange);
    ASSERT(dataSize);

    (void)bufferSize;
    (void)data;

    NTSTATUS status = STATUS_SUCCESS;
    return status;
}

// Private methods

/**
 * The capture pin's constructor.  Initialize any non-0, non-NULL fields
 * (since new will have zero'ed the memory anyway) and set up our
 * device level pointers for access during capture routines.
 * @param ksPin The AVStream pin object corresponding to the capture pin.
 */
CapturePin::CapturePin(KSPIN *ksPin)
{
    PAGED_CODE();

    DbgPrint("CapturePin::CapturePin\n");

    ASSERT(ksPin);

    m_ksPin = ksPin;
    KSDEVICE *ksDevice = ::KsPinGetDevice(ksPin);
    // Set up our device pointer.  This gives us access to "hardware I/O"
    // during the capture routines.
    m_device = reinterpret_cast<Device *> (ksDevice->Context);
}

CapturePin::~CapturePin()
{
    PAGED_CODE();
    //
}

/**
 * This is called when the caputre pin transitions state. The routine
 * attempts to acquire/release any hardware resources and start up
 * or shut down capture based on the states we are transitioning to
 * and away from.
 * @param toKsState The state we're transitioning to.
 * @param fromKsState The state we're transitioning away from.
 * @return NT status code.
 */
NTSTATUS
CapturePin::SetState(KSSTATE toKsState, KSSTATE fromKsState)
{
    PAGED_CODE();

    DbgPrint("CapturePin::SetState\n");

    (void)toKsState;
    (void)fromKsState;

    NTSTATUS status = STATUS_SUCCESS;
    return status;
}

/**
 * The process dispatch for the pin bridges to this location.
 * We handle setting up scatter gather mappings, etc...
 * @return NT status code.
 */
NTSTATUS
CapturePin::Process()
{
    PAGED_CODE();

    DbgPrint("CapturePin::Process\n");

    NTSTATUS status = STATUS_SUCCESS;
    return status;
}

/**
 * Clean up any references we're holding on frames after we abruptly
 * stop the hardware.
 * @return NT status code.
 */
NTSTATUS
CapturePin::CleanupReferences()
{
    PAGED_CODE();

    KSSTREAM_POINTER *clone = ::KsPinGetFirstCloneStreamPointer(m_ksPin);
    KSSTREAM_POINTER *nextClone = NULL;
    // Walk through the clones, deleting them, and setting DataUsed to
    // zero since we didn't use any data!
    while (NULL != clone) {

        nextClone = ::KsStreamPointerGetNextClone(clone);
        clone->StreamHeader->DataUsed = 0;
        ::KsStreamPointerDelete(clone);
        clone = nextClone;
    }

    return STATUS_SUCCESS;
}

// Private static methods

/**
 * This is the free callback from the bagged item (CapturePin). If we
 * do not provide a callback when we bag the CapturePin, ExFreePool
 * would be called. This is not desirable for C++ constructed objects.
 * We merely delete the object here.
 */
void
CapturePin::Cleanup(CapturePin *pin)
{
    PAGED_CODE();

    DbgPrint("CapturePin::Cleanup\n");

    delete pin;
}

/* DESCRIPTOR AND DISPATCH LAYOUT */

/**
 * This is the dispatch table for the capture pin. It provides notifications
 * about creation, closure, processing, data formats, etc...
 */
const
KSPIN_DISPATCH
ksCapturePinDispatch = {
    CapturePin::DispatchCreate,            // Pin Create
    NULL,                                  // Pin Close
    CapturePin::DispatchProcess,           // Pin Process
    NULL,                                  // Pin Reset
    CapturePin::DispatchSetFormat,         // Pin Set Data Format
    CapturePin::DispatchSetState,          // Pin Set Device State
    NULL,                                  // Pin Connect
    NULL,                                  // Pin Disconnect
    NULL,                                  // Clock Dispatch
    NULL                                   // Allocator Dispatch
};

/**
 *
 */
static
const
KSDATARANGE_AUDIO
ksAudioStreamFormat = {

    { // KSDATARANGE
      sizeof (KSDATARANGE_AUDIO),             // FormatSize
      0,                                      // Flags
      4,                                      // SampleSize
      0,                                      // Reserved
      STATIC_KSDATAFORMAT_TYPE_AUDIO,             // Media type
      STATIC_KSDATAFORMAT_SUBTYPE_PCM,            // Media subtype
      STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX, // Media format specifier
    },

    2,                                      // MaximumChannels
    16,                                     // MinimumBitsPerSample
    32,                                     // MaximumBitsPerSample
    48000,                                  // MinimumSampleFrequency
    48000                                   // MaximumSampleFrequency

};

/**
 *
 */
const
PKSDATARANGE
ksAudioPinDataRanges[1] = {
    PKSDATARANGE(&ksAudioStreamFormat)
};

/**
 * A default framing for the audio pin. In order for this to work properly,
 * the frame size must be at least 1/fps * bytes_per_sec large. Otherwise,
 * the audio stream will fall behind. This is dynamically adjusted when
 * the actual pin is created.
 */
DECLARE_SIMPLE_FRAMING_EX (
        ksAudioPinAllocatorFraming,                  //name
        STATICGUIDOF(KSMEMORY_TYPE_KERNEL_NONPAGED),   //memory type
        KSALLOCATOR_REQUIREMENTF_SYSTEM_MEMORY | KSALLOCATOR_REQUIREMENTF_PREFERENCES_ONLY, //flags
        25,             //frames
        0,              //alignment
        2 * PAGE_SIZE,  //min frame size
        2 * PAGE_SIZE   //max frame size
        );






//
// A GUID identifying the name of the audio capture pin.  I use the standard
// STATIC_PINNAME_VIDEO_CAPTURE for the video capture pin, but a custom name
// as defined in avssamp.inf for the audio capture pin.
//
//GUID g_PINNAME_AUDIO_CAPTURE =
//{0xba1184b9, 0x1fe6, 0x488a, 0xae, 0x78, 0x6e, 0x99, 0x7b, 0x2, 0xca, 0xea};












