/*
*/

#include "device.h"


#ifdef ALLOC_PRAGMA
#pragma code_seg("PAGE")
#endif // ALLOC_PRAGMA


/* Device */

// Public methods


// Public static methods

/**
 * Create the XXX device. This is the creation dispatch for the
 * capture device.
 * @param ksDeviceObject The AVStream device being created.
 * @return NT status code.
 */
NTSTATUS
Device::DispatchPnpAdd(KSDEVICE *ksDeviceObject)
{
    PAGED_CODE();

    DbgPrint("Device::DispatchPnpAdd\n");

    Device *device = new (NonPagedPool) Device(ksDeviceObject);
    if (NULL == device) {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    NTSTATUS status = device->PnpAdd();
    if (!NT_SUCCESS(status)) {
        delete device;
    }

    return status;
}

/**
 * This is the Pnp Start dispatch for XXX device. It simply
 * bridges to PnpStart() in the context of the Device.
 * @param ksDeviceObject
 * @param irp
 * @param translatedResourceList
 * @param untranslatedResourceList
 * @return NT status code.
 */
NTSTATUS
Device::DispatchPnpStart(KSDEVICE *ksDeviceObject, IRP *irp,
                         CM_RESOURCE_LIST *translatedResourceList,
                         CM_RESOURCE_LIST *untranslatedResourceList)
{
    PAGED_CODE();

    DbgPrint("Device::DispatchPnpStart\n");

    (void)irp;
    (void)translatedResourceList;
    (void)untranslatedResourceList;

    ASSERT(ksDeviceObject);
    ASSERT(ksDeviceObject->Context);
    Device *device = reinterpret_cast<Device *>(ksDeviceObject->Context);
    return device->PnpStart();
}

/**
 * This is the Pnp stop dispatch for the XXX device. It simply
 * bridges to PnpStop() in the context of the Device.
 * @param ksDeviceObject
 * @param irp
 */
void
Device::DispatchPnpStop (KSDEVICE *ksDeviceObject, IRP *irp)
{
    PAGED_CODE();

    DbgPrint("Device::DispatchPnpStop\n");

    (void)irp;

    ASSERT(ksDeviceObject);
    ASSERT(ksDeviceObject->Context);

    Device *device = reinterpret_cast<Device *>(ksDeviceObject->Context);
    device->PnpStop();
}

// Private methods

/**
 *
 */
Device::Device(KSDEVICE *ksDeviceObject)
{
    PAGED_CODE();

    DbgPrint("Device::Device\n");

    ASSERT(ksDeviceObject);
    m_ksDeviceObject = ksDeviceObject;
}

/**
 *
 */
Device::~Device()
{
}

/**
 *
 * @return NT status code.
 */
NTSTATUS
Device::PnpAdd()
{
    PAGED_CODE();

    DbgPrint("Device::PnpAdd\n");

    ::KsAcquireDevice(m_ksDeviceObject);
    NTSTATUS status = ::KsAddItemToObjectBag(m_ksDeviceObject->Bag,
                                             reinterpret_cast<PVOID>(this),
                                             reinterpret_cast<PFNKSFREE>(Device::Cleanup));
    ::KsReleaseDevice(m_ksDeviceObject);
    if (NT_SUCCESS(status)) {
        m_ksDeviceObject->Context = reinterpret_cast<PVOID>(this);
    }

    return status;
}

/**
 * This is the Pnp start routine for XXX.
 * @note That DispatchStart bridges to here in the context.
 * @return NT status code.
 */
NTSTATUS
Device::PnpStart()
{
    PAGED_CODE();

    DbgPrint("Device::PnpStart\n");

    NTSTATUS status = STATUS_SUCCESS;

    if (!m_ksDeviceObject->Started) {
        // Create the Filter for the device
        ::KsAcquireDevice(m_ksDeviceObject);
        status = ::KsCreateFilterFactory(m_ksDeviceObject->FunctionalDeviceObject,
                                          &ksCaptureFilterDescriptor,
                                          L"PcmCapture",
                                          NULL,
                                          KSCREATE_ITEM_FREEONSTOP,
                                          NULL,
                                          NULL,
                                          NULL);
        ::KsReleaseDevice(m_ksDeviceObject);
    }

    return status;
}

/**
 * This is the pnp stop dispatch for the XXX device. It releases any
 * adapter object previously allocated by IoGetDmaAdapter during Pnp Start.
 */
void
Device::PnpStop()
{
    PAGED_CODE();

    DbgPrint("Device::PnpStop\n");

    // Stub
}

// Private static methods

/**
 * This is the free callback for the bagged XXX device. Not providing
 * one will call ExFreePool, which is not what we want for a constructed
 * C++ object. This simply deletes the XXX device.
 * @param device
 */
void
Device::Cleanup(Device *device)
{
    DbgPrint("Device::Cleanup\n");

    delete device;
}

/* DESCRIPTOR AND DISPATCH LAYOUT */

/**
 * This is the dispatch table for the device. Plug and play
 * notifications as well as power management notifications are dispatched
 * through this table.
 */
static const
KSDEVICE_DISPATCH
ksDeviceDispatch = {
    Device::DispatchPnpAdd,    // Pnp Add Device
    Device::DispatchPnpStart,  // Pnp Start
    NULL,                      // Post-Start
    NULL,                      // Pnp Query Stop
    NULL,                      // Pnp Cancel Stop
    Device::DispatchPnpStop,   // PnP Stop
    NULL,                      // Pnp Query Remove
    NULL,                      // Pnp Cancel Remove
    NULL,                      // Pnp Remove
    NULL,                      // Pnp Query Capabilities
    NULL,                      // Pnp Surprise Removal
    NULL,                      // Power Query Power
    NULL,                      // Power Set Power
    NULL                       // Pnp Query Interface
};

/**
 * This is the device descriptor for the complete device. It points to the
 * dispatch table. No filter descriptors are statically in this descriptor.
 * This driver uses KsCreateFilterFactory to create the filters.
 */
const
KSDEVICE_DESCRIPTOR
ksDeviceDescriptor = {
    &ksDeviceDispatch,
    0,
    NULL,  // the filter descriptor is NULL, use KsCreateFilterFactory to create it dynamically
    KSDEVICE_DESCRIPTOR_VERSION
};


