/*
*/

#include "filter.h"
#include "capture.h"

#ifdef ALLOC_PRAGMA
#pragma code_seg("PAGE")
#endif // ALLOC_PRAGMA


/* CaptureFilter */

// Public methods


// Public static methods

/**
 * This is the filter creation dispatch for the capture filter. It
 * creates the CaptureFilter object, associates it with the AVStream
 * object, and bags it for easy cleanup later.
 * @param ksFilter The AVStream filter being created
 * @param irp The creation Irp
 * @return NT status code.
 */
NTSTATUS
CaptureFilter::DispatchCreate(KSFILTER *ksFilter, IRP *irp)
{
    PAGED_CODE();

    DbgPrint("CaptureFilter::DispatchCreate\n");

    (void)irp;

    CaptureFilter *capFilter = new (NonPagedPool) CaptureFilter(ksFilter);
    if (!capFilter) {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    NTSTATUS status = ::KsAddItemToObjectBag(ksFilter->Bag,
                                             reinterpret_cast<PVOID>(capFilter),
                                             reinterpret_cast<PFNKSFREE>(CaptureFilter::Cleanup));
    if (NT_SUCCESS(status)) {
        ksFilter->Context = reinterpret_cast<PVOID>(capFilter);
    } else {
        DbgPrint("CaptureFilter::DispatchCreate::KsAddItemToObjectBag - fail\n");
        delete capFilter;
    }

    return status;
}

// Private methods

/**
 * The capture filter object constructor.  Since the new operator will
 * have zeroed the memory, do not bother initializing any NULL or 0
 * fields. Only initialize non-NULL, non-0 fields.
 */
CaptureFilter::CaptureFilter(KSFILTER *ksFilter)
{
    PAGED_CODE();

    ASSERT(ksFilter);

    DbgPrint("CaptureFilter::CaptureFilter\n");

    m_ksFilter = ksFilter;
}

CaptureFilter::~CaptureFilter()
{
    PAGED_CODE();
}

// Private static methods

/**
 * This is the bag cleanup callback for the CaptureFilter. Not providing
 * one would cause ExFreePool to be used. This is not good for C++
 * constructed objects. We simply delete the object here.
 */
void
CaptureFilter::Cleanup(CaptureFilter *capFilter)
{
    PAGED_CODE();

    DbgPrint("CaptureFilter::Cleanup\n");

    delete capFilter;
}

/* DESCRIPTOR AND DISPATCH LAYOUT */

// {27C304E1-80F6-11d5-AFD4-0001032B6C32}
DEFINE_GUIDSTRUCT("27C304E1-80F6-11d5-AFD4-0001032B6C32", PINNAME_AUDIO_IN);
#define PINNAME_AUDIO_IN DEFINE_GUIDNAMED(PINNAME_AUDIO_IN)

// {27C304E1-80F6-11d5-AFD4-0001032B6C31}
DEFINE_GUIDSTRUCT("27C304E1-80F6-11d5-AFD4-0001032B6C31", PINNAME_AUDIO_OUT);
#define PINNAME_AUDIO_OUT DEFINE_GUIDNAMED(PINNAME_AUDIO_OUT)

/**
 *
 */
static
const
GUID
guidCaptureFilterCategories[] = {
    STATICGUIDOF(KSCATEGORY_AUDIO),
    STATICGUIDOF(KSCATEGORY_CAPTURE)
};

/**
 * The list of pin descriptors on the capture filter.
 */
static
const
KSPIN_DESCRIPTOR_EX
ksCaptureFilterPinDescriptors[] = {

    { // Audio Input Pin
      NULL,                                     // Dispatch
      NULL,                                     // AutomationTable
      { // PinDescriptor
        DEFINE_KSPIN_DEFAULT_INTERFACES,        // Interfaces
        DEFINE_KSPIN_DEFAULT_MEDIUMS,           // Mediums
        0,                                      // Range Count
        NULL,                                   // Ranges
        KSPIN_DATAFLOW_IN,                      // Dataflow
        KSPIN_COMMUNICATION_NONE,               // Communication
        &KSNODETYPE_LINE_CONNECTOR,             // Category
        reinterpret_cast<const GUID *>(&PINNAME_AUDIO_IN), // Name
        0                                       // Reserved
      },
      KSPIN_FLAG_DO_NOT_USE_STANDARD_TRANSPORT, // Flags
      0,                                        // Instances Possible
      0,                                        // Instances Necessary
      NULL,                                     // Allocator Framing
      NULL                                      // IntersectHandler
    },

    { //Audio Output Pin
      &ksCapturePinDispatch,                    // Dispatch
      NULL,                                     // AutomationTable
      { // PinDescriptor
        DEFINE_KSPIN_DEFAULT_INTERFACES,        // Interfaces
        DEFINE_KSPIN_DEFAULT_MEDIUMS,           // Mediums
        SIZEOF_ARRAY(ksAudioPinDataRanges),     // Range Count
        ksAudioPinDataRanges,                   // Ranges
        KSPIN_DATAFLOW_OUT,                     // Dataflow
        KSPIN_COMMUNICATION_BOTH,               // Communication (both source and sink)
        &KSCATEGORY_AUDIO,                      // Category
        reinterpret_cast<const GUID *>(&PINNAME_AUDIO_OUT), // Name
        0                                       // Reserved
      },
      KSPIN_FLAG_GENERATE_MAPPINGS | KSPIN_FLAG_DO_NOT_INITIATE_PROCESSING | KSPIN_FLAG_FIXED_FORMAT, // Flags
      1,                                        // Instances Possible
      0,                                        // Instances Necessary
      &ksAudioPinAllocatorFraming,              // Allocator Framing
      reinterpret_cast<PFNKSINTERSECTHANDLEREX>(CapturePin::IntersectHandler)
    }

};

/**
 * This is the dispatch table for the capture filter.  It provides notification
 * of creation, closure, processing (for filter-centrics, not for the capture
 * filter), and resets (for filter-centrics, not for the capture filter).
 */
static
const
KSFILTER_DISPATCH
ksCaptureFilterDispatch = {
    CaptureFilter::DispatchCreate,    // Filter Create
    NULL,                             // Filter Close
    NULL,                             // Filter Process
    NULL                              // Filter Reset
};


/**
 *
 */
static
const
KSNODE_DESCRIPTOR
ksAudioNodeDescriptors[] = {
    { //Node 0 : ADC
      NULL,               //Automation table
      &KSNODETYPE_ADC,    //Type (Analog-to-digital converter)
      NULL                //Name
    },

    { ////Node 1 : Volume
      NULL,               //Automation table
      &KSNODETYPE_VOLUME, //Type
      NULL                //Name
    }
};

typedef enum {
    ADC_NODE =0,
    VOLUME_NODE
} NODE_NAMES;

typedef enum {
    AUDIO_IN_PIN = 0,
    AUDIO_OUT_PIN
} PIN_NAMES;

/**
 *
 */
static
const
KSTOPOLOGY_CONNECTION
ksAudioConnections[] = {
    //FromNode       //FromNode Pin  //ToNode       //ToNodePin
    { KSFILTER_NODE, AUDIO_IN_PIN,   ADC_NODE,      1 /*out pin*/ },
    { ADC_NODE,      0, /*in pin*/   VOLUME_NODE,   1 /*out pin*/ },
    { VOLUME_NODE,   0, /*in pin*/   KSFILTER_NODE, AUDIO_OUT_PIN }
};


/**
 * The descriptor for the capture filter. We don't specify any topology
 * since there's only one pin on the filter. Realistically, there would
 * be some topological relationships here because there would be input
 * pins from crossbars and the like.
 */
const
KSFILTER_DESCRIPTOR
ksCaptureFilterDescriptor = {
    &ksCaptureFilterDispatch,               // Dispatch Table
    NULL,                                   // Automation Table
    KSFILTER_DESCRIPTOR_VERSION,            // Version
    0,                                      // Flags
    &KSNAME_Filter,                         // Reference GUID
    DEFINE_KSFILTER_PIN_DESCRIPTORS(ksCaptureFilterPinDescriptors),
    DEFINE_KSFILTER_CATEGORIES(guidCaptureFilterCategories),  // Categories
    DEFINE_KSFILTER_NODE_DESCRIPTORS(ksAudioNodeDescriptors), // NodeDescriptors
    DEFINE_KSFILTER_CONNECTIONS(ksAudioConnections),          // Connections
    NULL                                                      // Component ID
};


